import clr
clr.AddReference('IronPython.Wpf')
import wpf
from System.Windows.Controls import StackPanel
from System.IO import StreamReader
from System.Windows.Input import Keyboard
from System import Type, Tuple, Array, Double
from System.Windows.Threading import Dispatcher
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import Control, Keys
clr.AddReference("Trimble.Vce.Core")
from Trimble.Vce.Core import TransactMethodCall, ModelEvents
from Trimble.Vce.Core.Components import WorldView, Project, Layer, TextStyle
clr.AddReference("Trimble.Vce.Alignment")
from Trimble.Vce.Alignment.Linestring import Linestring,VerticalElementFactory, IVerticalNoCurve, IStationVerticalLocation
clr.AddReference("Trimble.Vce.Geometry")
from Trimble.Vce.Geometry import Point3D, Intersections
from Trimble.Vce.Geometry.PolySeg import PolySeg
clr.AddReference("Trimble.Vce.Interfaces")
from Trimble.Vce.Interfaces import Client
clr.AddReference("Trimble.Vce.UI.Controls")
#from Trimble.Vce.UI.Controls import MousePosition, CursorStyle
# later version of TBC moved these to a different assembly. If not found in new location, look at old
try:
    clr.AddReference("Trimble.Vce.ViewModel")
    clr.AddReference("Trimble.Vce.Units")
    from Trimble.CustomControl.Interfaces import MousePosition
    from Trimble.CustomControl.Interfaces.Enums import CursorStyle
except:
    from Trimble.Vce.UI.Controls import MousePosition, CursorStyle

from Trimble.Vce.UI.Controls.Wpf import CoordinateEdit, DistanceEdit
from Trimble.CustomControl.Interfaces.Enums import InputMethod

def Setup(cmdData, macroFileFolder):
    cmdData.Key = "VpiAtElev"
    cmdData.CommandName = "VpiAtElev"
    cmdData.Caption = "_Vpi At Elev"
    cmdData.UIForm = "VpiAtElev"
    cmdData.HelpFile = "Macros.chm"
    cmdData.HelpTopic = "22506"

    cmdData.DefaultTabKey = "Macros"
    cmdData.DefaultTabGroupKey = "CAD"
    cmdData.ShortCaption = "Vpi At Elev"
    cmdData.ToolTipTittle = "Vpi At Elev"
    cmdData.ToolTipTextFormatted = "Inserts VPI at every vertical intersection of line and plane."
    try:
        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass

class VpiAtElev(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\VpiAtElev.xaml") as s:
            wpf.LoadComponent(self, s)
        self.currentProject = currentProject

    def OnLoad(self, cmd, buttons, event):
        buttons[0].Content = "Apply"
        buttons[1].Content = "Close"
        self.Caption = cmd.Command.Caption
        self.lineCtl.AddTypeFilter(clr.GetClrType(Linestring))

    def CancelClicked(self, thisCmd, args):
        thisCmd.CloseUICommand()

    def OkClicked(self, thisCmd, e):
        self.textBoxCtl.Text= ""
        line = self.lineCtl.Entity
        if not line:
            self.textBoxCtl.Text= "No line selected"
            return
        elev = self.elevCtl.Distance
        if Double.IsNaN(elev):
            self.textBoxCtl.Text= "No elevation defined"
            return
        self.elevCtl.StatusMessage = ""
        # tell undo manager we are starting to do something
        self.currentProject.TransactionManager.AddBeginMark(Client.CommandGranularity.Command, self.Caption)

        try:
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
                poly=line.ComputeVerticalPolySeg()
                if not poly or poly.NumberOfNodes < 2:
                    self.textBoxCtl.Text= "Line not defined"
                    return
                begSta = poly.FirstNode.Point.X
                endSta = poly.LastNode.Point.X
                acrossPoly = PolySeg()
                acrossPoly.Add(Array[Point3D]([Point3D(begSta, elev), Point3D(endSta, elev)]))
                ints = Intersections()
                nFound = 0
                if poly.Intersect(acrossPoly, True, ints):
                    for i in ints:
                        e = VerticalElementFactory.Create(clr.GetClrType(IVerticalNoCurve), clr.GetClrType(IStationVerticalLocation))
                        e.StationDistance = i.Point.X
                        e.Elevation = elev
                        line.AddVerticalElement(e)
                        nFound += 1

                self.textBoxCtl.Text= nFound.ToString() + " Elevations defined"
                # you MUST commit the change to DB or they will all be un-done (roll-back)
                failGuard.Commit()
        finally:
            self.currentProject.TransactionManager.AddEndMark(Client.CommandGranularity.Command)

        Keyboard.Focus(self.lineCtl)
        return